/* AUTOGENERATED by gen-posix-conf-vars.awk.  DO NOT EDIT.  */

#ifndef _POSIX_THREAD_ATTR_STACKSIZE
# define CONF_DEF_POSIX_THREAD_ATTR_STACKSIZE CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_ATTR_STACKSIZE > 0
#  define CONF_DEF_POSIX_THREAD_ATTR_STACKSIZE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_ATTR_STACKSIZE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PRIORITY_SCHEDULING
# define CONF_DEF_POSIX_PRIORITY_SCHEDULING CONF_DEF_UNDEFINED
#else
# if _POSIX_PRIORITY_SCHEDULING > 0
#  define CONF_DEF_POSIX_PRIORITY_SCHEDULING CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PRIORITY_SCHEDULING CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PIPE
# define CONF_DEF_POSIX_PIPE CONF_DEF_UNDEFINED
#else
# if _POSIX_PIPE > 0
#  define CONF_DEF_POSIX_PIPE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PIPE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FILE_ATTRIBUTES
# define CONF_DEF_POSIX_FILE_ATTRIBUTES CONF_DEF_UNDEFINED
#else
# if _POSIX_FILE_ATTRIBUTES > 0
#  define CONF_DEF_POSIX_FILE_ATTRIBUTES CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FILE_ATTRIBUTES CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V6_ILP32_OFF32
# define CONF_DEF_POSIX_V6_ILP32_OFF32 CONF_DEF_UNDEFINED
#else
# if _POSIX_V6_ILP32_OFF32 > 0
#  define CONF_DEF_POSIX_V6_ILP32_OFF32 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V6_ILP32_OFF32 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_USER_GROUPS_R
# define CONF_DEF_POSIX_USER_GROUPS_R CONF_DEF_UNDEFINED
#else
# if _POSIX_USER_GROUPS_R > 0
#  define CONF_DEF_POSIX_USER_GROUPS_R CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_USER_GROUPS_R CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_USER_GROUPS
# define CONF_DEF_POSIX_USER_GROUPS CONF_DEF_UNDEFINED
#else
# if _POSIX_USER_GROUPS > 0
#  define CONF_DEF_POSIX_USER_GROUPS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_USER_GROUPS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SIGNALS
# define CONF_DEF_POSIX_SIGNALS CONF_DEF_UNDEFINED
#else
# if _POSIX_SIGNALS > 0
#  define CONF_DEF_POSIX_SIGNALS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SIGNALS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SHELL
# define CONF_DEF_POSIX_SHELL CONF_DEF_UNDEFINED
#else
# if _POSIX_SHELL > 0
#  define CONF_DEF_POSIX_SHELL CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SHELL CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_XTI
# define CONF_DEF_POSIX_PII_XTI CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_XTI > 0
#  define CONF_DEF_POSIX_PII_XTI CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_XTI CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_SOCKET
# define CONF_DEF_POSIX_PII_SOCKET CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_SOCKET > 0
#  define CONF_DEF_POSIX_PII_SOCKET CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_SOCKET CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MEMORY_PROTECTION
# define CONF_DEF_POSIX_MEMORY_PROTECTION CONF_DEF_UNDEFINED
#else
# if _POSIX_MEMORY_PROTECTION > 0
#  define CONF_DEF_POSIX_MEMORY_PROTECTION CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MEMORY_PROTECTION CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_BASE
# define CONF_DEF_POSIX_BASE CONF_DEF_UNDEFINED
#else
# if _POSIX_BASE > 0
#  define CONF_DEF_POSIX_BASE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_BASE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V7_LPBIG_OFFBIG
# define CONF_DEF_POSIX_V7_LPBIG_OFFBIG CONF_DEF_UNDEFINED
#else
# if _POSIX_V7_LPBIG_OFFBIG > 0
#  define CONF_DEF_POSIX_V7_LPBIG_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V7_LPBIG_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _XBS5_LP64_OFF64
# define CONF_DEF_XBS5_LP64_OFF64 CONF_DEF_UNDEFINED
#else
# if _XBS5_LP64_OFF64 > 0
#  define CONF_DEF_XBS5_LP64_OFF64 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_XBS5_LP64_OFF64 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TIMERS
# define CONF_DEF_POSIX_TIMERS CONF_DEF_UNDEFINED
#else
# if _POSIX_TIMERS > 0
#  define CONF_DEF_POSIX_TIMERS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TIMERS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TIMEOUTS
# define CONF_DEF_POSIX_TIMEOUTS CONF_DEF_UNDEFINED
#else
# if _POSIX_TIMEOUTS > 0
#  define CONF_DEF_POSIX_TIMEOUTS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TIMEOUTS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SYSTEM_DATABASE
# define CONF_DEF_POSIX_SYSTEM_DATABASE CONF_DEF_UNDEFINED
#else
# if _POSIX_SYSTEM_DATABASE > 0
#  define CONF_DEF_POSIX_SYSTEM_DATABASE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SYSTEM_DATABASE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SEMAPHORES
# define CONF_DEF_POSIX_SEMAPHORES CONF_DEF_UNDEFINED
#else
# if _POSIX_SEMAPHORES > 0
#  define CONF_DEF_POSIX_SEMAPHORES CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SEMAPHORES CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_DEVICE_IO
# define CONF_DEF_POSIX_DEVICE_IO CONF_DEF_UNDEFINED
#else
# if _POSIX_DEVICE_IO > 0
#  define CONF_DEF_POSIX_DEVICE_IO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_DEVICE_IO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V7_ILP32_OFFBIG
# define CONF_DEF_POSIX_V7_ILP32_OFFBIG CONF_DEF_UNDEFINED
#else
# if _POSIX_V7_ILP32_OFFBIG > 0
#  define CONF_DEF_POSIX_V7_ILP32_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V7_ILP32_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V6_LP64_OFF64
# define CONF_DEF_POSIX_V6_LP64_OFF64 CONF_DEF_UNDEFINED
#else
# if _POSIX_V6_LP64_OFF64 > 0
#  define CONF_DEF_POSIX_V6_LP64_OFF64 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V6_LP64_OFF64 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TRACE_INHERIT
# define CONF_DEF_POSIX_TRACE_INHERIT CONF_DEF_UNDEFINED
#else
# if _POSIX_TRACE_INHERIT > 0
#  define CONF_DEF_POSIX_TRACE_INHERIT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TRACE_INHERIT CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_PROCESS_SHARED
# define CONF_DEF_POSIX_THREAD_PROCESS_SHARED CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_PROCESS_SHARED > 0
#  define CONF_DEF_POSIX_THREAD_PROCESS_SHARED CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_PROCESS_SHARED CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_PRIORITY_SCHEDULING
# define CONF_DEF_POSIX_THREAD_PRIORITY_SCHEDULING CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_PRIORITY_SCHEDULING > 0
#  define CONF_DEF_POSIX_THREAD_PRIORITY_SCHEDULING CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_PRIORITY_SCHEDULING CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_STREAM_MAX
# define CONF_DEF_POSIX_STREAM_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_STREAM_MAX > 0
#  define CONF_DEF_POSIX_STREAM_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_STREAM_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_CHILD_MAX
# define CONF_DEF_POSIX_CHILD_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_CHILD_MAX > 0
#  define CONF_DEF_POSIX_CHILD_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_CHILD_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_ADVISORY_INFO
# define CONF_DEF_POSIX_ADVISORY_INFO CONF_DEF_UNDEFINED
#else
# if _POSIX_ADVISORY_INFO > 0
#  define CONF_DEF_POSIX_ADVISORY_INFO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_ADVISORY_INFO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_IPV6
# define CONF_DEF_POSIX_IPV6 CONF_DEF_UNDEFINED
#else
# if _POSIX_IPV6 > 0
#  define CONF_DEF_POSIX_IPV6 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_IPV6 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SYSTEM_DATABASE_R
# define CONF_DEF_POSIX_SYSTEM_DATABASE_R CONF_DEF_UNDEFINED
#else
# if _POSIX_SYSTEM_DATABASE_R > 0
#  define CONF_DEF_POSIX_SYSTEM_DATABASE_R CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SYSTEM_DATABASE_R CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SHARED_MEMORY_OBJECTS
# define CONF_DEF_POSIX_SHARED_MEMORY_OBJECTS CONF_DEF_UNDEFINED
#else
# if _POSIX_SHARED_MEMORY_OBJECTS > 0
#  define CONF_DEF_POSIX_SHARED_MEMORY_OBJECTS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SHARED_MEMORY_OBJECTS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_REGEXP
# define CONF_DEF_POSIX_REGEXP CONF_DEF_UNDEFINED
#else
# if _POSIX_REGEXP > 0
#  define CONF_DEF_POSIX_REGEXP CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_REGEXP CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_OSI_CLTS
# define CONF_DEF_POSIX_PII_OSI_CLTS CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_OSI_CLTS > 0
#  define CONF_DEF_POSIX_PII_OSI_CLTS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_OSI_CLTS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_NGROUPS_MAX
# define CONF_DEF_POSIX_NGROUPS_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_NGROUPS_MAX > 0
#  define CONF_DEF_POSIX_NGROUPS_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_NGROUPS_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MAPPED_FILES
# define CONF_DEF_POSIX_MAPPED_FILES CONF_DEF_UNDEFINED
#else
# if _POSIX_MAPPED_FILES > 0
#  define CONF_DEF_POSIX_MAPPED_FILES CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MAPPED_FILES CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FILE_SYSTEM
# define CONF_DEF_POSIX_FILE_SYSTEM CONF_DEF_UNDEFINED
#else
# if _POSIX_FILE_SYSTEM > 0
#  define CONF_DEF_POSIX_FILE_SYSTEM CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FILE_SYSTEM CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_DEVICE_SPECIFIC_R
# define CONF_DEF_POSIX_DEVICE_SPECIFIC_R CONF_DEF_UNDEFINED
#else
# if _POSIX_DEVICE_SPECIFIC_R > 0
#  define CONF_DEF_POSIX_DEVICE_SPECIFIC_R CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_DEVICE_SPECIFIC_R CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_BARRIERS
# define CONF_DEF_POSIX_BARRIERS CONF_DEF_UNDEFINED
#else
# if _POSIX_BARRIERS > 0
#  define CONF_DEF_POSIX_BARRIERS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_BARRIERS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TRACE_LOG
# define CONF_DEF_POSIX_TRACE_LOG CONF_DEF_UNDEFINED
#else
# if _POSIX_TRACE_LOG > 0
#  define CONF_DEF_POSIX_TRACE_LOG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TRACE_LOG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_SPORADIC_SERVER
# define CONF_DEF_POSIX_THREAD_SPORADIC_SERVER CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_SPORADIC_SERVER > 0
#  define CONF_DEF_POSIX_THREAD_SPORADIC_SERVER CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_SPORADIC_SERVER CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_ATTR_STACKADDR
# define CONF_DEF_POSIX_THREAD_ATTR_STACKADDR CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_ATTR_STACKADDR > 0
#  define CONF_DEF_POSIX_THREAD_ATTR_STACKADDR CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_ATTR_STACKADDR CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SPAWN
# define CONF_DEF_POSIX_SPAWN CONF_DEF_UNDEFINED
#else
# if _POSIX_SPAWN > 0
#  define CONF_DEF_POSIX_SPAWN CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SPAWN CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PRIORITIZED_IO
# define CONF_DEF_POSIX_PRIORITIZED_IO CONF_DEF_UNDEFINED
#else
# if _POSIX_PRIORITIZED_IO > 0
#  define CONF_DEF_POSIX_PRIORITIZED_IO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PRIORITIZED_IO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MEMLOCK_RANGE
# define CONF_DEF_POSIX_MEMLOCK_RANGE CONF_DEF_UNDEFINED
#else
# if _POSIX_MEMLOCK_RANGE > 0
#  define CONF_DEF_POSIX_MEMLOCK_RANGE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MEMLOCK_RANGE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V6_LPBIG_OFFBIG
# define CONF_DEF_POSIX_V6_LPBIG_OFFBIG CONF_DEF_UNDEFINED
#else
# if _POSIX_V6_LPBIG_OFFBIG > 0
#  define CONF_DEF_POSIX_V6_LPBIG_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V6_LPBIG_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_RAW_SOCKETS
# define CONF_DEF_POSIX_RAW_SOCKETS CONF_DEF_UNDEFINED
#else
# if _POSIX_RAW_SOCKETS > 0
#  define CONF_DEF_POSIX_RAW_SOCKETS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_RAW_SOCKETS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SAVED_IDS
# define CONF_DEF_POSIX_SAVED_IDS CONF_DEF_UNDEFINED
#else
# if _POSIX_SAVED_IDS > 0
#  define CONF_DEF_POSIX_SAVED_IDS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SAVED_IDS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_OSI_M
# define CONF_DEF_POSIX_PII_OSI_M CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_OSI_M > 0
#  define CONF_DEF_POSIX_PII_OSI_M CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_OSI_M CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_OSI_COTS
# define CONF_DEF_POSIX_PII_OSI_COTS CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_OSI_COTS > 0
#  define CONF_DEF_POSIX_PII_OSI_COTS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_OSI_COTS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_NETWORKING
# define CONF_DEF_POSIX_NETWORKING CONF_DEF_UNDEFINED
#else
# if _POSIX_NETWORKING > 0
#  define CONF_DEF_POSIX_NETWORKING CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_NETWORKING CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_DEVICE_SPECIFIC
# define CONF_DEF_POSIX_DEVICE_SPECIFIC CONF_DEF_UNDEFINED
#else
# if _POSIX_DEVICE_SPECIFIC > 0
#  define CONF_DEF_POSIX_DEVICE_SPECIFIC CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_DEVICE_SPECIFIC CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_CPUTIME
# define CONF_DEF_POSIX_CPUTIME CONF_DEF_UNDEFINED
#else
# if _POSIX_CPUTIME > 0
#  define CONF_DEF_POSIX_CPUTIME CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_CPUTIME CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_ASYNCHRONOUS_IO
# define CONF_DEF_POSIX_ASYNCHRONOUS_IO CONF_DEF_UNDEFINED
#else
# if _POSIX_ASYNCHRONOUS_IO > 0
#  define CONF_DEF_POSIX_ASYNCHRONOUS_IO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_ASYNCHRONOUS_IO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V7_LP64_OFF64
# define CONF_DEF_POSIX_V7_LP64_OFF64 CONF_DEF_UNDEFINED
#else
# if _POSIX_V7_LP64_OFF64 > 0
#  define CONF_DEF_POSIX_V7_LP64_OFF64 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V7_LP64_OFF64 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TZNAME_MAX
# define CONF_DEF_POSIX_TZNAME_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_TZNAME_MAX > 0
#  define CONF_DEF_POSIX_TZNAME_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TZNAME_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TRACE_EVENT_FILTER
# define CONF_DEF_POSIX_TRACE_EVENT_FILTER CONF_DEF_UNDEFINED
#else
# if _POSIX_TRACE_EVENT_FILTER > 0
#  define CONF_DEF_POSIX_TRACE_EVENT_FILTER CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TRACE_EVENT_FILTER CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_PRIO_PROTECT
# define CONF_DEF_POSIX_THREAD_PRIO_PROTECT CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_PRIO_PROTECT > 0
#  define CONF_DEF_POSIX_THREAD_PRIO_PROTECT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_PRIO_PROTECT CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SPORADIC_SERVER
# define CONF_DEF_POSIX_SPORADIC_SERVER CONF_DEF_UNDEFINED
#else
# if _POSIX_SPORADIC_SERVER > 0
#  define CONF_DEF_POSIX_SPORADIC_SERVER CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SPORADIC_SERVER CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SPIN_LOCKS
# define CONF_DEF_POSIX_SPIN_LOCKS CONF_DEF_UNDEFINED
#else
# if _POSIX_SPIN_LOCKS > 0
#  define CONF_DEF_POSIX_SPIN_LOCKS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SPIN_LOCKS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_READER_WRITER_LOCKS
# define CONF_DEF_POSIX_READER_WRITER_LOCKS CONF_DEF_UNDEFINED
#else
# if _POSIX_READER_WRITER_LOCKS > 0
#  define CONF_DEF_POSIX_READER_WRITER_LOCKS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_READER_WRITER_LOCKS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_POLL
# define CONF_DEF_POSIX_POLL CONF_DEF_UNDEFINED
#else
# if _POSIX_POLL > 0
#  define CONF_DEF_POSIX_POLL CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_POLL CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_OSI
# define CONF_DEF_POSIX_PII_OSI CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_OSI > 0
#  define CONF_DEF_POSIX_PII_OSI CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_OSI CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_INTERNET_DGRAM
# define CONF_DEF_POSIX_PII_INTERNET_DGRAM CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_INTERNET_DGRAM > 0
#  define CONF_DEF_POSIX_PII_INTERNET_DGRAM CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_INTERNET_DGRAM CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII
# define CONF_DEF_POSIX_PII CONF_DEF_UNDEFINED
#else
# if _POSIX_PII > 0
#  define CONF_DEF_POSIX_PII CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V7_ILP32_OFF32
# define CONF_DEF_POSIX_V7_ILP32_OFF32 CONF_DEF_UNDEFINED
#else
# if _POSIX_V7_ILP32_OFF32 > 0
#  define CONF_DEF_POSIX_V7_ILP32_OFF32 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V7_ILP32_OFF32 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _XBS5_LPBIG_OFFBIG
# define CONF_DEF_XBS5_LPBIG_OFFBIG CONF_DEF_UNDEFINED
#else
# if _XBS5_LPBIG_OFFBIG > 0
#  define CONF_DEF_XBS5_LPBIG_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_XBS5_LPBIG_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_SAFE_FUNCTIONS
# define CONF_DEF_POSIX_THREAD_SAFE_FUNCTIONS CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_SAFE_FUNCTIONS > 0
#  define CONF_DEF_POSIX_THREAD_SAFE_FUNCTIONS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_SAFE_FUNCTIONS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SYNCHRONIZED_IO
# define CONF_DEF_POSIX_SYNCHRONIZED_IO CONF_DEF_UNDEFINED
#else
# if _POSIX_SYNCHRONIZED_IO > 0
#  define CONF_DEF_POSIX_SYNCHRONIZED_IO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SYNCHRONIZED_IO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_REALTIME_SIGNALS
# define CONF_DEF_POSIX_REALTIME_SIGNALS CONF_DEF_UNDEFINED
#else
# if _POSIX_REALTIME_SIGNALS > 0
#  define CONF_DEF_POSIX_REALTIME_SIGNALS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_REALTIME_SIGNALS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FSYNC
# define CONF_DEF_POSIX_FSYNC CONF_DEF_UNDEFINED
#else
# if _POSIX_FSYNC > 0
#  define CONF_DEF_POSIX_FSYNC CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FSYNC CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _XBS5_ILP32_OFFBIG
# define CONF_DEF_XBS5_ILP32_OFFBIG CONF_DEF_UNDEFINED
#else
# if _XBS5_ILP32_OFFBIG > 0
#  define CONF_DEF_XBS5_ILP32_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_XBS5_ILP32_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MULTI_PROCESS
# define CONF_DEF_POSIX_MULTI_PROCESS CONF_DEF_UNDEFINED
#else
# if _POSIX_MULTI_PROCESS > 0
#  define CONF_DEF_POSIX_MULTI_PROCESS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MULTI_PROCESS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MESSAGE_PASSING
# define CONF_DEF_POSIX_MESSAGE_PASSING CONF_DEF_UNDEFINED
#else
# if _POSIX_MESSAGE_PASSING > 0
#  define CONF_DEF_POSIX_MESSAGE_PASSING CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MESSAGE_PASSING CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FILE_LOCKING
# define CONF_DEF_POSIX_FILE_LOCKING CONF_DEF_UNDEFINED
#else
# if _POSIX_FILE_LOCKING > 0
#  define CONF_DEF_POSIX_FILE_LOCKING CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FILE_LOCKING CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FD_MGMT
# define CONF_DEF_POSIX_FD_MGMT CONF_DEF_UNDEFINED
#else
# if _POSIX_FD_MGMT > 0
#  define CONF_DEF_POSIX_FD_MGMT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FD_MGMT CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_CLOCK_SELECTION
# define CONF_DEF_POSIX_CLOCK_SELECTION CONF_DEF_UNDEFINED
#else
# if _POSIX_CLOCK_SELECTION > 0
#  define CONF_DEF_POSIX_CLOCK_SELECTION CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_CLOCK_SELECTION CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TYPED_MEMORY_OBJECTS
# define CONF_DEF_POSIX_TYPED_MEMORY_OBJECTS CONF_DEF_UNDEFINED
#else
# if _POSIX_TYPED_MEMORY_OBJECTS > 0
#  define CONF_DEF_POSIX_TYPED_MEMORY_OBJECTS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TYPED_MEMORY_OBJECTS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREADS
# define CONF_DEF_POSIX_THREADS CONF_DEF_UNDEFINED
#else
# if _POSIX_THREADS > 0
#  define CONF_DEF_POSIX_THREADS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREADS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_PRIO_INHERIT
# define CONF_DEF_POSIX_THREAD_PRIO_INHERIT CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_PRIO_INHERIT > 0
#  define CONF_DEF_POSIX_THREAD_PRIO_INHERIT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_PRIO_INHERIT CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SSIZE_MAX
# define CONF_DEF_POSIX_SSIZE_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_SSIZE_MAX > 0
#  define CONF_DEF_POSIX_SSIZE_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SSIZE_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_INTERNET_STREAM
# define CONF_DEF_POSIX_PII_INTERNET_STREAM CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_INTERNET_STREAM > 0
#  define CONF_DEF_POSIX_PII_INTERNET_STREAM CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_INTERNET_STREAM CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_OPEN_MAX
# define CONF_DEF_POSIX_OPEN_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_OPEN_MAX > 0
#  define CONF_DEF_POSIX_OPEN_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_OPEN_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MONOTONIC_CLOCK
# define CONF_DEF_POSIX_MONOTONIC_CLOCK CONF_DEF_UNDEFINED
#else
# if _POSIX_MONOTONIC_CLOCK > 0
#  define CONF_DEF_POSIX_MONOTONIC_CLOCK CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MONOTONIC_CLOCK CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_V6_ILP32_OFFBIG
# define CONF_DEF_POSIX_V6_ILP32_OFFBIG CONF_DEF_UNDEFINED
#else
# if _POSIX_V6_ILP32_OFFBIG > 0
#  define CONF_DEF_POSIX_V6_ILP32_OFFBIG CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_V6_ILP32_OFFBIG CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_VERSION
# define CONF_DEF_POSIX_VERSION CONF_DEF_UNDEFINED
#else
# if _POSIX_VERSION > 0
#  define CONF_DEF_POSIX_VERSION CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_VERSION CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_CPUTIME
# define CONF_DEF_POSIX_THREAD_CPUTIME CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_CPUTIME > 0
#  define CONF_DEF_POSIX_THREAD_CPUTIME CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_CPUTIME CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SINGLE_PROCESS
# define CONF_DEF_POSIX_SINGLE_PROCESS CONF_DEF_UNDEFINED
#else
# if _POSIX_SINGLE_PROCESS > 0
#  define CONF_DEF_POSIX_SINGLE_PROCESS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SINGLE_PROCESS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_PII_INTERNET
# define CONF_DEF_POSIX_PII_INTERNET CONF_DEF_UNDEFINED
#else
# if _POSIX_PII_INTERNET > 0
#  define CONF_DEF_POSIX_PII_INTERNET CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_PII_INTERNET CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_JOB_CONTROL
# define CONF_DEF_POSIX_JOB_CONTROL CONF_DEF_UNDEFINED
#else
# if _POSIX_JOB_CONTROL > 0
#  define CONF_DEF_POSIX_JOB_CONTROL CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_JOB_CONTROL CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_FIFO
# define CONF_DEF_POSIX_FIFO CONF_DEF_UNDEFINED
#else
# if _POSIX_FIFO > 0
#  define CONF_DEF_POSIX_FIFO CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_FIFO CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_ARG_MAX
# define CONF_DEF_POSIX_ARG_MAX CONF_DEF_UNDEFINED
#else
# if _POSIX_ARG_MAX > 0
#  define CONF_DEF_POSIX_ARG_MAX CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_ARG_MAX CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _XBS5_ILP32_OFF32
# define CONF_DEF_XBS5_ILP32_OFF32 CONF_DEF_UNDEFINED
#else
# if _XBS5_ILP32_OFF32 > 0
#  define CONF_DEF_XBS5_ILP32_OFF32 CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_XBS5_ILP32_OFF32 CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_THREAD_DESTRUCTOR_ITERATIONS
# define CONF_DEF_POSIX_THREAD_DESTRUCTOR_ITERATIONS CONF_DEF_UNDEFINED
#else
# if _POSIX_THREAD_DESTRUCTOR_ITERATIONS > 0
#  define CONF_DEF_POSIX_THREAD_DESTRUCTOR_ITERATIONS CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_THREAD_DESTRUCTOR_ITERATIONS CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_TRACE
# define CONF_DEF_POSIX_TRACE CONF_DEF_UNDEFINED
#else
# if _POSIX_TRACE > 0
#  define CONF_DEF_POSIX_TRACE CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_TRACE CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_SELECT
# define CONF_DEF_POSIX_SELECT CONF_DEF_UNDEFINED
#else
# if _POSIX_SELECT > 0
#  define CONF_DEF_POSIX_SELECT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_SELECT CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_MEMLOCK
# define CONF_DEF_POSIX_MEMLOCK CONF_DEF_UNDEFINED
#else
# if _POSIX_MEMLOCK > 0
#  define CONF_DEF_POSIX_MEMLOCK CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_MEMLOCK CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_C_LANG_SUPPORT_R
# define CONF_DEF_POSIX_C_LANG_SUPPORT_R CONF_DEF_UNDEFINED
#else
# if _POSIX_C_LANG_SUPPORT_R > 0
#  define CONF_DEF_POSIX_C_LANG_SUPPORT_R CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_C_LANG_SUPPORT_R CONF_DEF_DEFINED_UNSET
# endif
#endif

#ifndef _POSIX_C_LANG_SUPPORT
# define CONF_DEF_POSIX_C_LANG_SUPPORT CONF_DEF_UNDEFINED
#else
# if _POSIX_C_LANG_SUPPORT > 0
#  define CONF_DEF_POSIX_C_LANG_SUPPORT CONF_DEF_DEFINED_SET
# else
#  define CONF_DEF_POSIX_C_LANG_SUPPORT CONF_DEF_DEFINED_UNSET
# endif
#endif

#if NEED_SPEC_ARRAY
static const struct { const char *name; int num; } specs[] =
  {
    { "POSIX_V6_ILP32_OFF32", _SC_V6_ILP32_OFF32 },
    { "POSIX_V7_LPBIG_OFFBIG", _SC_V7_LPBIG_OFFBIG },
    { "POSIX_V6_LP64_OFF64", _SC_V6_LP64_OFF64 },
    { "POSIX_V7_ILP32_OFFBIG", _SC_V7_ILP32_OFFBIG },
    { "XBS5_LP64_OFF64", _SC_XBS5_LP64_OFF64 },
    { "POSIX_V6_LPBIG_OFFBIG", _SC_V6_LPBIG_OFFBIG },
    { "POSIX_V7_LP64_OFF64", _SC_V7_LP64_OFF64 },
    { "POSIX_V7_ILP32_OFF32", _SC_V7_ILP32_OFF32 },
    { "XBS5_LPBIG_OFFBIG", _SC_XBS5_LPBIG_OFFBIG },
    { "XBS5_ILP32_OFFBIG", _SC_XBS5_ILP32_OFFBIG },
    { "POSIX_V6_ILP32_OFFBIG", _SC_V6_ILP32_OFFBIG },
    { "XBS5_ILP32_OFF32", _SC_XBS5_ILP32_OFF32 },
  };
static const size_t nspecs = sizeof (specs) / sizeof (specs[0]);
#endif
