/* AUTOGENERATED by gen-tunables.awk.  */
#ifndef _TUNABLES_H_
# error "Do not include this file directly."
# error "Include tunables.h instead."
#endif

typedef enum
{
  TUNABLE_ENUM_NAME(glibc, cpu, hwcaps),
  TUNABLE_ENUM_NAME(glibc, cpu, plt_rewrite),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_data_cache_size),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_ibt),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_memset_non_temporal_threshold),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_non_temporal_threshold),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_rep_movsb_threshold),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_rep_stosb_threshold),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_shared_cache_size),
  TUNABLE_ENUM_NAME(glibc, cpu, x86_shstk),
  TUNABLE_ENUM_NAME(glibc, gmon, maxarcs),
  TUNABLE_ENUM_NAME(glibc, gmon, minarcs),
  TUNABLE_ENUM_NAME(glibc, malloc, arena_max),
  TUNABLE_ENUM_NAME(glibc, malloc, arena_test),
  TUNABLE_ENUM_NAME(glibc, malloc, check),
  TUNABLE_ENUM_NAME(glibc, malloc, hugetlb),
  TUNABLE_ENUM_NAME(glibc, malloc, mmap_max),
  TUNABLE_ENUM_NAME(glibc, malloc, mmap_threshold),
  TUNABLE_ENUM_NAME(glibc, malloc, mxfast),
  TUNABLE_ENUM_NAME(glibc, malloc, perturb),
  TUNABLE_ENUM_NAME(glibc, malloc, tcache_count),
  TUNABLE_ENUM_NAME(glibc, malloc, tcache_max),
  TUNABLE_ENUM_NAME(glibc, malloc, tcache_unsorted_limit),
  TUNABLE_ENUM_NAME(glibc, malloc, top_pad),
  TUNABLE_ENUM_NAME(glibc, malloc, trim_threshold),
  TUNABLE_ENUM_NAME(glibc, mem, decorate_maps),
  TUNABLE_ENUM_NAME(glibc, mem, tagging),
  TUNABLE_ENUM_NAME(glibc, pthread, mutex_spin_count),
  TUNABLE_ENUM_NAME(glibc, pthread, rseq),
  TUNABLE_ENUM_NAME(glibc, pthread, stack_cache_size),
  TUNABLE_ENUM_NAME(glibc, pthread, stack_hugetlb),
  TUNABLE_ENUM_NAME(glibc, rtld, dynamic_sort),
  TUNABLE_ENUM_NAME(glibc, rtld, enable_secure),
  TUNABLE_ENUM_NAME(glibc, rtld, execstack),
  TUNABLE_ENUM_NAME(glibc, rtld, nns),
  TUNABLE_ENUM_NAME(glibc, rtld, optional_static_tls),
} tunable_id_t;


#ifdef TUNABLES_INTERNAL
# define TUNABLE_NAME_MAX 44
# define TUNABLE_ALIAS_MAX 23
# include "dl-tunable-types.h"
static tunable_t tunable_list[] attribute_relro __attribute_used__ = {
  {TUNABLE_NAME_S(glibc, cpu, hwcaps), {TUNABLE_TYPE_STRING, 0, 0}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, plt_rewrite), {TUNABLE_TYPE_INT_32, 0, 2}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_data_cache_size), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_ibt), {TUNABLE_TYPE_STRING, 0, 0}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_memset_non_temporal_threshold), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_non_temporal_threshold), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_rep_movsb_threshold), {TUNABLE_TYPE_SIZE_T, 1, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_rep_stosb_threshold), {TUNABLE_TYPE_SIZE_T, 1, SIZE_MAX}, {.numval = 2048}, {.numval = 2048}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_shared_cache_size), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, cpu, x86_shstk), {TUNABLE_TYPE_STRING, 0, 0}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, gmon, maxarcs), {TUNABLE_TYPE_INT_32, 50, INT32_MAX}, {.numval = 1048576}, {.numval = 1048576}, false, {0}},
  {TUNABLE_NAME_S(glibc, gmon, minarcs), {TUNABLE_TYPE_INT_32, 50, INT32_MAX}, {.numval = 50}, {.numval = 50}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, arena_max), {TUNABLE_TYPE_SIZE_T, 1, SIZE_MAX}, {}, {}, false, "MALLOC_ARENA_MAX"},
  {TUNABLE_NAME_S(glibc, malloc, arena_test), {TUNABLE_TYPE_SIZE_T, 1, SIZE_MAX}, {}, {}, false, "MALLOC_ARENA_TEST"},
  {TUNABLE_NAME_S(glibc, malloc, check), {TUNABLE_TYPE_INT_32, 0, 3}, {}, {}, false, "MALLOC_CHECK_"},
  {TUNABLE_NAME_S(glibc, malloc, hugetlb), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, mmap_max), {TUNABLE_TYPE_INT_32, 0, INT32_MAX}, {}, {}, false, "MALLOC_MMAP_MAX_"},
  {TUNABLE_NAME_S(glibc, malloc, mmap_threshold), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, "MALLOC_MMAP_THRESHOLD_"},
  {TUNABLE_NAME_S(glibc, malloc, mxfast), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, perturb), {TUNABLE_TYPE_INT_32, 0, 0xff}, {}, {}, false, "MALLOC_PERTURB_"},
  {TUNABLE_NAME_S(glibc, malloc, tcache_count), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, tcache_max), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, tcache_unsorted_limit), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, malloc, top_pad), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {.numval = 131072}, {.numval = 131072}, false, "MALLOC_TOP_PAD_"},
  {TUNABLE_NAME_S(glibc, malloc, trim_threshold), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {}, {}, false, "MALLOC_TRIM_THRESHOLD_"},
  {TUNABLE_NAME_S(glibc, mem, decorate_maps), {TUNABLE_TYPE_INT_32, 0, 1}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, mem, tagging), {TUNABLE_TYPE_INT_32, 0, 255}, {}, {}, false, {0}},
  {TUNABLE_NAME_S(glibc, pthread, mutex_spin_count), {TUNABLE_TYPE_INT_32, 0, 32767}, {.numval = 100}, {.numval = 100}, false, {0}},
  {TUNABLE_NAME_S(glibc, pthread, rseq), {TUNABLE_TYPE_INT_32, 0, 1}, {.numval = 1}, {.numval = 1}, false, {0}},
  {TUNABLE_NAME_S(glibc, pthread, stack_cache_size), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {.numval = 41943040}, {.numval = 41943040}, false, {0}},
  {TUNABLE_NAME_S(glibc, pthread, stack_hugetlb), {TUNABLE_TYPE_INT_32, 0, 1}, {.numval = 1}, {.numval = 1}, false, {0}},
  {TUNABLE_NAME_S(glibc, rtld, dynamic_sort), {TUNABLE_TYPE_INT_32, 1, 2}, {.numval = 2}, {.numval = 2}, false, {0}},
  {TUNABLE_NAME_S(glibc, rtld, enable_secure), {TUNABLE_TYPE_INT_32, 0, 1}, {.numval = 0}, {.numval = 0}, false, {0}},
  {TUNABLE_NAME_S(glibc, rtld, execstack), {TUNABLE_TYPE_INT_32, 0, 2}, {.numval = 1}, {.numval = 1}, false, {0}},
  {TUNABLE_NAME_S(glibc, rtld, nns), {TUNABLE_TYPE_SIZE_T, 1, 16}, {.numval = 4}, {.numval = 4}, false, {0}},
  {TUNABLE_NAME_S(glibc, rtld, optional_static_tls), {TUNABLE_TYPE_SIZE_T, 0, SIZE_MAX}, {.numval = 512}, {.numval = 512}, false, {0}},
};

static const tunable_id_t tunable_env_alias_list[] =
{
  TUNABLE_ENUM_NAME(glibc, malloc, arena_max),
  TUNABLE_ENUM_NAME(glibc, malloc, arena_test),
  TUNABLE_ENUM_NAME(glibc, malloc, check),
  TUNABLE_ENUM_NAME(glibc, malloc, mmap_max),
  TUNABLE_ENUM_NAME(glibc, malloc, mmap_threshold),
  TUNABLE_ENUM_NAME(glibc, malloc, perturb),
  TUNABLE_ENUM_NAME(glibc, malloc, top_pad),
  TUNABLE_ENUM_NAME(glibc, malloc, trim_threshold),
};
#endif
